@@ -1,3 +1,4 @@
+#!/usr/bin/env python
 """
 This is use to drive many of the examples across the image backends
 and is used for regression testing and comparing backend efficiency
@@ -17,25 +18,39 @@ files = (
     'barchart_demo.py',
     'color_demo.py',
     'csd_demo.py',
+    'date_demo1.py',
+    'date_demo2.py',    
+    'fill_demo.py',
     'figtext.py',
+    'finance_demo.py',
     'histogram_demo.py',
+    'image_demo.py',
+    'image_demo2.py',
+    'image_demo_na.py',
+    'invert_axes.py',
     'legend_demo.py',
     'legend_demo2.py',
     'line_styles.py',
     'log_demo.py',
     'log_test.py',
-    'mri_with_eeg.py',
+    'mathtext_demo.py',
+#    'mri_with_eeg.py',
+    'major_minor_demo1.py',
+    'major_minor_demo2.py',     
     'multiple_figs_demo.py',
     'pcolor_demo.py',
+    'pcolor_demo2.py',
     'psd_demo.py',
     'scatter_demo.py',
     'scatter_demo2.py',
     'simple_plot.py',
     'stock_demo.py',
+    'specgram_demo.py',
     'subplot_demo.py',
     'table_demo.py',
     'text_handles.py',
     'text_themes.py',
+    'two_scales.py',
     )
 
 def drive(backend):
@@ -49,23 +64,24 @@ def drive(backend):
         for line in file(fname):
             if line.strip().startswith('from __future__ import division'): continue
             if line.strip().startswith('matplotlib.use'): continue
-            if line.strip().startswith('#savefig'): continue
+            if line.strip().startswith('savefig'): continue
             if line.strip().startswith('show'): continue
             lines.append(line)
         basename, ext = os.path.splitext(fname)
         outfile = basename + '_%s'%backend
-        if backend in ('GTK', 'WX'):
+        if backend in ('GTK', 'WX', 'TkAgg'):
             lines.append('show()')
         else:
-            lines.append('#savefig("%s", dpi=150)' % outfile)
+            lines.append('savefig("%s", dpi=150)' % outfile)
         tmpfile = '_tmp_%s.py' % basename
         file(tmpfile, 'w').write(''.join(lines))
         os.system('python %s' % tmpfile)
 
 times = {}
 backends = ['PS', 'GD', 'Paint', 'Agg', 'Template']
-backends.extend([ 'GTK', 'WX'])
+#backends.extend([ 'GTK', 'WX', 'TkAgg'])
 #backends = [ 'Agg']
+#backends = [ 'Agg', 'PS', 'Template']
 
 for backend in backends:
     print 'testing %s' % backend
