@@ -1,4 +1,5 @@
-from Numeric import arange, sin, pi
+#!/usr/bin/env python
+from matplotlib.numerix import arange, sin, pi
 
 import matplotlib
 matplotlib.use('GTK')
@@ -19,12 +20,12 @@ win.add(vbox)
 vbox.show()
 
 f = Figure(figsize=(5,4), dpi=100)
-a = Subplot(f, 111)
+a = f.add_subplot(111)
 t = arange(0.0,3.0,0.01)
 s = sin(2*pi*t)
 
 a.plot(t,s)
-f.add_axis(a)
+
 
 canvas = FigureCanvasGTK(f)  # a gtk.DrawingArea
 canvas.show()
