@@ -20,11 +20,15 @@ elif backend=='Paint':
          error_msg, new_figure_manager
 elif backend=='Agg':
     from backend_agg import new_figure_manager
+elif backend=='SVG':
+    from backend_svg import new_figure_manager
 elif backend=='GTKGD':
     from backend_gtkgd import \
          error_msg, draw_if_interactive, show, new_figure_manager
 elif backend=='GTKAgg':
     from backend_gtkagg import error_msg, draw_if_interactive, show, new_figure_manager
+elif backend=='TkAgg':
+    from backend_tkagg import error_msg, draw_if_interactive, show, new_figure_manager
 elif backend=='PS':
     from backend_ps import new_figure_manager
 elif backend=='GD':
@@ -32,6 +36,9 @@ elif backend=='GD':
 elif backend=='WX':
     from backend_wx import \
          error_msg, draw_if_interactive, show, new_figure_manager, Toolbar
+elif backend=='WXAgg':
+    from backend_wxagg import \
+         error_msg, draw_if_interactive, show, new_figure_manager, Toolbar
 else:
     raise ValueError, 'Unrecognized backend %s' % backend
     
