@@ -16,20 +16,33 @@ from gtk import gdk
 import pango
 
 import matplotlib
-from Numeric import Int16, asarray
+from matplotlib.numerix import asarray, fromstring, UInt8, zeros, \
+     where, transpose, nonzero, indices, max, ones, nx
 
 from matplotlib.cbook import is_string_like,  enumerate, True, False
-from matplotlib.transforms import Bound2D
+from matplotlib.font_manager import fontManager
 
 from matplotlib.backend_bases import \
      RendererBase, GraphicsContextBase, FigureManagerBase, FigureCanvasBase
 from matplotlib._matlab_helpers import Gcf
 from matplotlib.figure import Figure
 
+# make if 0 to disable mathtext
+if 1:
+    try:
+        from matplotlib.mathtext import math_parse_s_ft2font
+    except ImportError:
+        print >>sys.stderr, 'backend_gtk could not import mathtext (build with ft2font)'
+        useMathText = False
+    else:
+        useMathText = True
+else:
+    useMathText = False
 
 # the true dots per inch on the screen; should be display dependent
 # see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
-PIXELS_PER_INCH = 72
+PIXELS_PER_INCH = 96
+
 
 def round(x):
     return int(math.floor(x+0.5))
@@ -70,17 +83,29 @@ colorManager = ColorManagerGTK()
 
 class RendererGTK(RendererBase):
 
-    fontweights = {'normal' : pango.WEIGHT_NORMAL,
-                   'bold' : pango.WEIGHT_BOLD,
-                   'heavy' : pango.WEIGHT_HEAVY,
-                   'light' : pango.WEIGHT_LIGHT,
-                   'normal' : pango.WEIGHT_NORMAL,
-                   'ultrabold' : pango.WEIGHT_ULTRABOLD,
-                   'ultralight' : pango.WEIGHT_ULTRALIGHT,
+    fontweights = {
+        100          : pango.WEIGHT_ULTRALIGHT,
+        200          : pango.WEIGHT_LIGHT,
+        300          : pango.WEIGHT_LIGHT,
+        400          : pango.WEIGHT_NORMAL,
+        500          : pango.WEIGHT_NORMAL,
+        600          : pango.WEIGHT_BOLD,
+        700          : pango.WEIGHT_BOLD,
+        800          : pango.WEIGHT_HEAVY,
+        900          : pango.WEIGHT_ULTRABOLD,
+        'ultralight' : pango.WEIGHT_ULTRALIGHT,
+        'light'      : pango.WEIGHT_LIGHT,
+        'normal'     : pango.WEIGHT_NORMAL,
+        'medium'     : pango.WEIGHT_NORMAL,
+        'semibold'   : pango.WEIGHT_BOLD,
+        'bold'       : pango.WEIGHT_BOLD,
+        'heavy'      : pango.WEIGHT_HEAVY,
+        'ultrabold'  : pango.WEIGHT_ULTRABOLD,
+        'black'      : pango.WEIGHT_ULTRABOLD,
                    }
     fontangles = {
-        'italic': pango.STYLE_ITALIC,
-        'normal' : pango.STYLE_NORMAL,
+        'italic'  : pango.STYLE_ITALIC,
+        'normal'  : pango.STYLE_NORMAL,
         'oblique' : pango.STYLE_OBLIQUE,
         }
 
@@ -89,56 +114,41 @@ class RendererGTK(RendererBase):
     extentd = {}  # a map from text prop tups to text extents
     offsetd = {}  # a map from text prop tups to text offsets
     rotated = {}  # a map from text prop tups to rotated text pixbufs
+
     def __init__(self, gtkDA, gdkDrawable, dpi):
         self.gtkDA = gtkDA
         self.gdkDrawable = gdkDrawable
         self.width, self.height = self.gdkDrawable.get_size()
         self.dpi = dpi
 
-    def compute_text_offsets(self, t, props=None):
-        """
-        Return the (x,y) offsets to adjust for the alignment
-        specifications
-        """
+    def flipy(self):
+        return True
 
-        if props is None:
-            props = t.get_prop_tup()
-        tup = self.offsetd.get(props)
-        if tup is not None: return tup
-        layout = self.get_pango_layout(t)
+    def offset_text_height(self):
+        return True
 
+    def get_text_width_height(self, s, prop, ismath):
+        """
+        get the width and height in display coords of the string s
+        with FontPropertry prop
+        """
+        if ismath:
+            width, height, fonts = math_parse_s_ft2font(
+                s, self.dpi.get(), prop.get_size_in_points())
+            return width, height
+
+        layout = self.get_pango_layout(s, prop)
         inkRect, logicalRect = layout.get_pixel_extents()
         rect = inkRect
+        #rect = logicalRect
         l, b, w, h = rect
+        return w, h+1
 
+    def get_canvas_width_height(self):
+        'return the canvas width and height in display coords'
+        return self.width, self.height
 
-
-        rotation = t.get_rotation()
-        halign = t.get_horizontalalignment()
-        valign = t.get_verticalalignment()
-        
-        if rotation=='vertical':
-            w, h = h, w
-            l,b = b, l
-            if halign=='center': offsetx = -w/2
-            elif halign=='right': offsetx = -w
-            else: offsetx = -b
-            
-            if valign=='center': offsety = -h/2-b
-            elif valign=='top': offsety = 0
-            else: offsety = -h
-        else:
-            if halign=='center': offsetx = -w/2
-            elif halign=='right': offsetx = -w
-            else: offsetx = 0
-
-            if valign=='center': offsety = h/2
-            elif valign=='top': offsety = 0
-            else: offsety = h
-
-        tup = offsetx-l, offsety+b
-        self.offsetd[props] = tup
-        return tup
+    
 
     
     def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):
@@ -156,6 +166,30 @@ class RendererGTK(RendererBase):
             self.gdkDrawable.draw_arc(gc.gdkGC, True, x, y, w, h, a1, a2)
             gc.gdkGC.foreground = edgecolor
         self.gdkDrawable.draw_arc(gc.gdkGC, False, x, y, w, h, a1, a2)
+
+    def draw_image(self, x, y, im):
+        """
+        Draw the Image instance into the current axes; x, y is the
+        upper left hand corner of the image
+        """
+        #print 'draw_image'
+
+        rows, cols, s = im.as_str()
+        X = fromstring(s, UInt8)
+        X.shape = cols, rows, 4
+        pb=gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
+                          has_alpha=1, bits_per_sample=8,
+                          width=rows, height=cols)
+        try: pa = pb.get_pixels_array()
+        except AttributeError: pa = pb.pixel_array
+
+        pa[:,:,:] = X
+
+        gc = self.new_gc()
+        pb.render_to_drawable(self.gdkDrawable, gc.gdkGC, 0, 0,
+                              int(x), int(self.height-y), rows, cols,
+                              gdk.RGB_DITHER_NONE, 0, 0)
+
             
     def draw_line(self, gc, x1, y1, x2, y2):
         """
@@ -167,8 +201,9 @@ class RendererGTK(RendererBase):
 
 
     def draw_lines(self, gc, x, y):
-        y = self.height - y.astype(Int16)
-        x = x.astype(Int16)
+        x = x.astype(nx.Int16)
+        y = self.height*ones(y.shape, nx.Int16) - y.astype(nx.Int16)  
+
         self.gdkDrawable.draw_lines(gc.gdkGC, zip(x,y))
 
     def draw_point(self, gc, x, y):
@@ -195,7 +230,7 @@ class RendererGTK(RendererBase):
             self.gdkDrawable.draw_polygon(gc.gdkGC, True, points)
             gc.gdkGC.foreground = edgecolor
 
-        self.gdkDrawable.draw_polygon(gc.gdkGC, True, points)
+        self.gdkDrawable.draw_polygon(gc.gdkGC, False, points)
 
 
     def draw_rectangle(self, gc, rgbFace, x, y, width, height):
@@ -204,10 +239,8 @@ class RendererGTK(RendererBase):
         If filled=True, fill the rectangle with the gc foreground
         gc is a GraphicsContext instance
         """
-
-        #x, y = int(x), self.height-int(y+height)
-        #w, h = int(width), int(height)
-        x, y = int(x), self.height-int(math.ceil(y+height))
+        x, y = int(x), self.height-int(y+height)
+        #x, y = int(x), self.height-int(math.ceil(y+height))
         w, h = int(math.ceil(width)), int(math.ceil(height))
 
 
@@ -221,7 +254,7 @@ class RendererGTK(RendererBase):
         self.gdkDrawable.draw_rectangle(gc.gdkGC, False, x, y, w, h)
 
 
-    def _draw_rotated_text(self, x, y, t, layout, gc, props):
+    def _draw_rotated_text(self, gc, x, y, s, prop, angle, ismath):
         """
         Draw the text rotated 90 degrees
         """
@@ -229,44 +262,45 @@ class RendererGTK(RendererBase):
         gdrawable = self.gdkDrawable
         ggc = gc.gdkGC
 
-
+        layout = self.get_pango_layout(s, prop)
         inkRect, logicalRect = layout.get_pixel_extents()
-        rect = logicalRect
-        w, h = int(rect[2]), int(rect[3])
+        rect = inkRect
+        l, b, w, h = rect
 
+        x = int(x-h)
+        y = int(y-w)
         # get the background image
-        ox, oy = self.compute_text_offsets(t, props)
-        y = self.height-y
-
-        xox = max(0,int(x+ox))
-        yoy = max(0,int(y+oy))
 
         # todo: cache rotation for dynamic redraw until pygtk mem leak
         # fixed
-        imageOut = self.rotated.get(props)
+        key = (x,y,s,angle,hash(prop))
+        imageOut = self.rotated.get(key)
         if imageOut is not None:
-            gdrawable.draw_image(ggc, imageOut, 0, 0, xox, yoy, h, w)
+            gdrawable.draw_image(ggc, imageOut, 0, 0, x, y, h, w)
             return
 
         # save the background
-        imageBack = gdrawable.get_image(xox, yoy, w, h)
-        imageVert = gdrawable.get_image(xox, yoy, h, w)
+        imageBack = gdrawable.get_image(x, y, w, h)
+        imageVert = gdrawable.get_image(x, y, h, w)
 
         # transform the vertical image, write it onto the renderer,
         # and draw the layout onto it
         imageFlip = gtk.gdk.Image(type=gdk.IMAGE_NORMAL,
                                   visual=gdrawable.get_visual(),
                                   width=w, height=h)
+        if imageFlip is None or imageBack is None or imageVert is None:
+            print >> sys.stderr, "Could not renderer vertical text", s
+            return
         imageFlip.set_colormap(gdrawable.get_colormap())
         for i in range(w):
             for j in range(h):
                 imageFlip.put_pixel(i, j, imageVert.get_pixel(j,w-i-1) )
 
-        gdrawable.draw_image(ggc, imageFlip, 0, 0, xox, yoy, w, h)
-        gdrawable.draw_layout(ggc, xox, yoy, layout)
+        gdrawable.draw_image(ggc, imageFlip, 0, 0, x, y, w, h)
+        gdrawable.draw_layout(ggc, x, y-b, layout)
 
         # now get that image and flip it vertical
-        imageIn = gdrawable.get_image(xox, yoy, w, h)
+        imageIn = gdrawable.get_image(x, y, w, h)
         imageOut = gtk.gdk.Image(type=gdk.IMAGE_NORMAL,
                                  visual=gdrawable.get_visual(),
                                  width=h, height=w)
@@ -276,96 +310,122 @@ class RendererGTK(RendererBase):
                 imageOut.put_pixel(j, i, imageIn.get_pixel(w-i-1,j) )
 
         # draw the old background and the flipped text
-        gdrawable.draw_image(ggc, imageBack, 0, 0, xox, yoy, w, h)
-        gdrawable.draw_image(ggc, imageOut, 0, 0, xox, yoy, h, w)
-        self.rotated[props] = imageOut
+        gdrawable.draw_image(ggc, imageBack, 0, 0, x, y, w, h)
+        gdrawable.draw_image(ggc, imageOut, 0, 0, x, y, h, w)
+        self.rotated[key] = imageOut
         return True
 
 
-    def draw_text(self, gc, x, y, t):
-        if t.get_text()=='': return
+    def draw_mathtext(self, gc, x, y, s, prop, angle):
 
-        layout = self.get_pango_layout(t)
+        size = prop.get_size_in_points()
+        width, height, fonts = math_parse_s_ft2font(
+            s, self.dpi.get(), size)
 
-        props = t.get_prop_tup()
-        ox, oy = self.compute_text_offsets(t, props)
+        x = int(x)
+        y = int(y)
+        
+        rgb = gc.get_rgb()
+        rgba = (rgb[0], rgb[1], rgb[2], gc.get_alpha())        
 
-        if t.get_rotation()=='vertical':            
-            drawn = self._draw_rotated_text(x, y, t, layout, gc, props)
-        else:
-            xox = int(x+ox)
-            yoy = self.height - int(y+oy)
+        imw, imh, s = fonts[0].image_as_str()
+        N = imw*imh
 
-            self.gdkDrawable.draw_layout(
-                gc.gdkGC, x=xox, y=yoy,
-                layout=layout)
+        # a numpixels by num fonts array
+        Xall = zeros((N,len(fonts)), typecode=UInt8)
+
+        for i, font in enumerate(fonts):
+            imw, imh, s = font.image_as_str()
+            Xall[:,i] = fromstring(s, UInt8)  
+
+        # get the max alpha at each pixel
+        Xs = max(Xall,1)
+
+        # convert it to it's proper shape
+        Xs.shape = imh, imw
+                
+
+        imw = min([imw, self.width-x])
+        imh = min([imh, y])
+
+        pb=gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
+                          has_alpha=1, bits_per_sample=8, width=imw, height=imh)
+
+        try: pbpix = pb.get_pixels_array()
+        except AttributeError: pbpix = pb.pixel_array
+
+
+        pbpix[:,:,0]=int(rgb[0]*255)
+        pbpix[:,:,1]=int(rgb[1]*255)
+        pbpix[:,:,2]=int(rgb[2]*255)
+        pbpix[:,:,3]=Xs
+        
+        pb.render_to_drawable(self.gdkDrawable, gc.gdkGC, 0, 0,
+                              x, int(y-height), imw, imh,
+                              gdk.RGB_DITHER_NONE, 0, 0)
+
+        
+    def draw_text(self, gc, x, y, s, prop, angle, ismath):
+
+        
+        if ismath:
+            return self.draw_mathtext(gc, x, y, s, prop, angle)
+
+        w, h = self.get_text_width_height(s, prop, ismath)
+        x = int(x)
+        y = int(y)
+
+        if angle==90:
+            self._draw_rotated_text(gc, x, y, s, prop, angle, ismath)
+            return
+        layout = self.get_pango_layout(s, prop)
+        inkRect, logicalRect = layout.get_pixel_extents()
+        rect = inkRect
+        l, b, w, h = rect
+
+        self.gdkDrawable.draw_layout(
+            gc.gdkGC, x=x, y=y-h-b,
+            layout=layout)
 
 
         
-    def get_pango_layout(self, t):
+    def get_pango_layout(self, s, prop):
         """
         Return a pango layout instance for Text instance t.  cache to
         layoutd
         """
-        props = t.get_prop_tup()
-        layout = self.layoutd.get(props)
+
+        key = self.dpi.get(), s, hash(prop)
+        layout = self.layoutd.get(key)
         if layout is not None: return layout
 
-        fontname = t.get_fontname()
-        if fontname.lower()=='times': fontname = "serif"
-            
-        font = pango.FontDescription(
-            '%s' % fontname)
-        font.set_weight(self.fontweights[t.get_fontweight()])
-        font.set_style(self.fontangles[t.get_fontangle()])
+        fontname = prop.get_name()
+        #if fontname.lower()=='times': fontname = "serif"
+        
+        font = pango.FontDescription('%s' % fontname)
+        
+        font.set_weight(self.fontweights[prop.get_weight()])
+        font.set_style(self.fontangles[prop.get_style()])
 
-        scale = self.get_text_scale(t)
-        font.set_size(int(scale*t.get_fontsize()*1024))
+        scale = self.get_text_scale()
+        size  = prop.get_size_in_points()
+        font.set_size(int(scale*size*1024))
         context = self.gtkDA.create_pango_context()
-        layout  = self.gtkDA.create_pango_layout(t.get_text())
+        layout  = self.gtkDA.create_pango_layout(s)
         layout.set_font_description(font)    
 
-        self.layoutd[props] = layout
+        self.layoutd[key] = layout
         return layout
 
 
-    def get_text_extent(self, t):
-        'Return the text extent in display coords as a Bound2D instance'
 
-        props = t.get_prop_tup()
-        extent = self.extentd.get(props)
-
-        if extent is not None: return extent
-
-        layout = self.get_pango_layout(t)
-        
-        inkRect, logicalRect = layout.get_pixel_extents()
-        rect = inkRect
-        l,b,w,h = rect
-        x, y  = t.get_xy_display()
-        ox, oy = self.compute_text_offsets(t, props)
-        x = x+ox
-        y = y+oy
-
-        if t.get_rotation()=='vertical':
-            w, h = h, w
-            left = x
-            bottom = y
-            left = x+b
-        else:
-            left = x+l
-            bottom = y-h-b
-
-        extent =  Bound2D(left, bottom, w, h)
-        self.extentd[props] = extent
-        return extent
-
-    def get_text_scale(self, t):
+    def get_text_scale(self):
         """
         Return the scale factor for fontsize taking screendpi and pixels per
         inch into account
         """
-        return t.dpi.get()/PIXELS_PER_INCH
+        return self.dpi.get()/PIXELS_PER_INCH
+        #return self.dpi.get()/72.0
 
     def new_gc(self):
         return GraphicsContextGTK(self.gdkDrawable.new_gc(), self)
@@ -375,7 +435,8 @@ class RendererGTK(RendererBase):
         convert point measures to pixes using dpi and the pixels per
         inch of the display
         """
-        return points*PIXELS_PER_INCH/72.0*self.dpi.get()/72.0
+        return points*(PIXELS_PER_INCH/72.0*self.dpi.get()/72.0)
+        #return points*(self.dpi.get()/72.0)
 
 
 class GraphicsContextGTK(GraphicsContextBase):
@@ -402,8 +463,10 @@ class GraphicsContextGTK(GraphicsContextBase):
     def set_clip_rectangle(self, rectangle):
         GraphicsContextBase.set_clip_rectangle(self, rectangle)
         l,b,w,h = rectangle
-        rectangle = (int(l), self.renderer.height-int(b+h),
+        rectangle = (int(l), self.renderer.height-int(b+h)+1,
                      int(w), int(h))
+        #rectangle = (int(l), self.renderer.height-int(b+h),
+        #             int(w+1), int(h+2))
         self.gdkGC.set_clip_rectangle(rectangle)        
 
 
@@ -507,7 +570,7 @@ def draw_if_interactive():
             #fig.queue_draw()
 
 
-def show():
+def show(mainloop=True):
     """
     Show all the figures and enter the gtk mainloop
 
@@ -517,7 +580,7 @@ def show():
     for manager in Gcf.get_all_fig_managers():
         manager.window.show()
         
-    gtk.mainloop()
+    if mainloop: gtk.mainloop()
 
 
 def _quit_after_print_xvfb(*args):
@@ -538,11 +601,11 @@ def show_xvfb():
     gtk.mainloop()                
 
 
-def new_figure_manager(num, *args):
+def new_figure_manager(num, *args, **kwargs):
     """
     Create a new figure manager instance
     """
-    figure = Figure(*args)
+    figure = Figure(*args, **kwargs)
 
     canvas = FigureCanvasGTK(figure)
     return FigureManagerGTK(canvas, num)
@@ -558,10 +621,10 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         self._gpixmap = None
         self._doplot = True
         self._printQued = []
+        self._idleID = 0 
 
-
-        w = figure.bbox.x.interval()
-        h = figure.bbox.y.interval()
+        w = figure.bbox.width()
+        h = figure.bbox.height()
         self.set_size_request(int(w), int(h))
         self.set_double_buffered(gtk.FALSE)
         self.connect('key_press_event', self.key_press_event)
@@ -597,8 +660,11 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         w,h = widget.window.get_size()
         if w==1 or h==1: return # empty fig
 
-        self.figure.bbox.set_bounds(0, 0, w, h)
-        for a in self.figure.axes: a.resize()
+        # compute desired figure size in inches
+        dpival = self.figure.dpi.get()
+        winch = w/dpival
+        hinch = h/dpival
+        self.figure.set_figsize_inches(winch, hinch)
         
         if self._isConfigured and self._isRealized:
             self.draw()
@@ -612,8 +678,8 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         drawable = self.window        
         if drawable is None: return
         
-        width = int(self.figure.bbox.x.interval())
-        height = int(self.figure.bbox.y.interval())
+        width = int(self.figure.bbox.width())
+        height = int(self.figure.bbox.height())
 
         gpixmap = gtk.gdk.Pixmap( drawable, width, height)
         pixmap = RendererGTK(self, gpixmap, self.figure.dpi)
@@ -627,6 +693,7 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         self._ggc = ggc
 
     def expose_event(self, widget, event):
+
         if widget.window is None: return 
 
         if self._gpixmap is not None:
@@ -676,30 +743,22 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
             origDPI = self.figure.dpi.get()
             ps = self.switch_backends(FigureCanvasPS)
             ps.figure.dpi.set(72)
-            for a in ps.figure.axes: a.resize()
             ps.print_figure(filename, 72, facecolor, edgecolor)
             self.figure.dpi.set(origDPI)
-            for a in self.figure.axes: a.resize()
             return
 
+        origDPI = self.figure.dpi.get()
         origfacecolor = self.figure.get_facecolor()
         origedgecolor = self.figure.get_edgecolor()
+
+        self.figure.dpi.set(dpi)        
         self.figure.set_facecolor(facecolor)
         self.figure.set_edgecolor(edgecolor)
 
-        origBounds = self.figure.bbox.get_bounds()
-        origDPI = self.figure.dpi.get()
-        dpi = 72.0/PIXELS_PER_INCH*dpi
-
-        self.figure.dpi.set(dpi)        
-        width = int(self.figure.figsize[0]*dpi)
-        height = int(self.figure.figsize[1]*dpi)
-        
-        self.figure.bbox.set_bounds(0, 0, width, height)
-
-
-        for a in self.figure.axes: a.resize()
+        l,b,width, height = self.figure.bbox.get_bounds()
 
+        width = int(width)
+        height = int(height)
         gdrawable = self.window
         ggc = gdrawable.new_gc()
 
@@ -712,13 +771,13 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8, width, height)
         pixbuf.get_from_drawable(gpixmap, gdrawable.get_colormap(),
                                  0, 0, 0, 0, width, height)
-
+        
         self.figure.set_facecolor(origfacecolor)
         self.figure.set_edgecolor(origedgecolor)
         self.figure.dpi.set(origDPI)
-        self.figure.bbox.set_bounds(*origBounds)
-        self.configure_event(self, 'configure')
 
+        self.configure_event(self, 'configure')
+            
         try: pixbuf.save(filename, type)
         except gobject.GError, msg:
             msg = raise_msg_to_str(msg)
@@ -794,8 +853,8 @@ class FigureManagerGTK(FigureManagerBase):
         self.toolbar.update()
         return a
     
-    def add_axes(self, rect, axisbg):
-        a = FigureManagerBase.add_axes(self, rect, axisbg)
+    def add_axes(self, rect, **kwargs):
+        a = FigureManagerBase.add_axes(self, rect, **kwargs)
         self.toolbar.update()
         return a
     
@@ -856,7 +915,8 @@ class NavigationToolbar(gtk.Toolbar):
         
         """
         gtk.Toolbar.__init__(self)
-        
+
+
         self.canvas = canvas
         self.win = window
         
@@ -970,31 +1030,31 @@ class NavigationToolbar(gtk.Toolbar):
 
         self.append_space()
 
-
-        def destroy_clicked(button):
-            win = self.win
-            if win is not None: win.destroy()
-            else: gtk.mainquit()
-
-        iconw = gtk.Image()
-        iconw.set_from_stock(gtk.STOCK_QUIT, iconSize)
-        self.bQuit = self.append_item(
-            'Close',
-            'Close the figure',
-            'Private',
-            iconw,
-            destroy_clicked)
-
-        self.append_space()
-
+        self.omenu = gtk.OptionMenu()
+        self.omenu.set_border_width(3)
+        self.insert_widget(
+            self.omenu,
+            'Select axes that controls affect',
+            'Private', 0)
+        
         self.update()
 
     def make_axis_menu(self):
 
+        menu = gtk.Menu()
+
+        label = "All"
+        self.itemAll = gtk.MenuItem(label)
+        menu.append(self.itemAll)
+
+        label = "Invert"
+        self.itemInvert = gtk.MenuItem(label)
+        menu.append(self.itemInvert)
+
         def toggled(item, label):
-            if item==itemAll:
+            if item==self.itemAll:
                 for item in items: item.set_active(1)
-            elif item==itemInvert:
+            elif item==self.itemInvert:
                 for item in items:
                     item.set_active(not item.get_active())
 
@@ -1004,19 +1064,11 @@ class NavigationToolbar(gtk.Toolbar):
 
                         
             
-        menu = gtk.Menu()
+        self.itemAll.connect("activate", toggled, label)
+        self.itemAll.show()
 
-        label = "All"
-        itemAll = gtk.MenuItem(label)
-        menu.append(itemAll)
-        itemAll.connect("activate", toggled, label)
-        itemAll.show()
-
-        label = "Invert"
-        itemInvert = gtk.MenuItem(label)
-        menu.append(itemInvert)
-        itemInvert.connect("activate", toggled, label)
-        itemInvert.show()
+        self.itemInvert.connect("activate", toggled, label)
+        self.itemInvert.show()
 
         items = []
         for i in range(len(self._axes)):
@@ -1029,6 +1081,7 @@ class NavigationToolbar(gtk.Toolbar):
             item.set_active(1)
             items.append(item)
 
+        self.omenuItems = items
         return menu
 
     def set_active(self, ind):
@@ -1118,19 +1171,9 @@ class NavigationToolbar(gtk.Toolbar):
     def update(self):
         self._axes = self.canvas.figure.axes
         self.set_active(range(len(self._axes)))
-        if len(self._axes)>1:
-
-            try: self.omenu
-            except AttributeError:
-                self.omenu = gtk.OptionMenu()
-                self.omenu.set_border_width(3)
-                self.omenu.show()
-                self.insert_widget(
-                    self.omenu,
-                    'Select axes that controls affect',
-                    'Private', 0)
-                
+        if len(self._axes)>1:                
             # set up the axis menu
+            self.omenu.show()
             menu = self.make_axis_menu()
             self.omenu.set_menu(menu)
         self.set_active(range(len(self._axes)))
