@@ -20,6 +20,75 @@ from __future__ import division
 # CVS log information
 #
 # $Log: backend_wx.py,v $
+# Revision 1.62  2004/05/21 14:41:32  jdh2358
+# added frameon for figure
+#
+# Revision 1.61  2004/05/14 18:23:35  jdh2358
+# new transformation / artist constructors
+#
+# Revision 1.60  2004/05/09 21:13:25  jodonoghue
+# Minor bugfixes: navigation toolbar now shows on MacOSX and quit operates correctly after last frame closed on all platforms.
+#
+# Revision 1.59  2004/05/07 14:48:47  jdh2358
+# fixed ismath sig across backends
+#
+# Revision 1.58  2004/05/04 18:37:43  jdh2358
+# major overhaul of text
+#
+# Revision 1.57  2004/05/03 16:18:42  jdh2358
+# removed close buttons on all GUIs per Steves advice
+#
+# Revision 1.56  2004/04/17 22:24:28  jdh2358
+# multiple scales on same axes
+#
+# Revision 1.55  2004/04/16 22:49:33  jdh2358
+# fixed wx exception swallowing bug
+#
+# Revision 1.54  2004/04/16 21:37:07  jdh2358
+# renamed coords demo
+#
+# Revision 1.53  2004/04/16 19:30:01  barrett
+# Minor fixes to GTK and WX backends to work with fonts_demo example.
+#
+# Revision 1.51  2004/04/05 12:58:54  jdh2358
+# removed debug info from wxagg
+#
+# Revision 1.50  2004/04/04 20:10:31  jdh2358
+# added prototype wxagg backend
+#
+# Revision 1.49  2004/03/16 20:26:39  jdh2358
+# sync to 0.52 release
+#
+# Revision 1.48  2004/03/08 22:17:21  jdh2358
+#
+# * Fixed embedding_in_wx and dynamic_demo_wx examples
+#
+# * Ported build to darwin
+#
+# * Tk:
+#
+#   removed default figman=None from nav toolbar since it needs the
+#   figman
+#
+#   fixed close bug
+#
+#   small changes to aid darwin build
+#
+# Revision 1.47  2004/03/08 14:56:52  jdh2358
+# fixed subplot bug
+#
+# Revision 1.46  2004/03/03 19:27:43  jdh2358
+# fixed gtk vert text bug
+#
+# Revision 1.45  2004/02/27 21:46:32  jdh2358
+# close to porting mathtext to gtk
+#
+# Revision 1.44  2004/02/27 14:47:48  jdh2358
+# added matplotlibrc and tk win32 compile fixes
+#
+# Revision 1.43  2004/02/23 16:20:56  jdh2358
+# resynced cvs to freetype2
+#
 # Revision 1.40  2004/02/16 18:05:01  jdh2358
 # fexed wx to work on windows and linux
 #
@@ -194,7 +263,7 @@ Examples which work on this release:
  (3) - Clipping seems to be broken.
 """
 
-cvs_id = '$Id: backend_wx.py,v 1.40 2004/02/16 18:05:01 jdh2358 Exp $'
+cvs_id = '$Id: backend_wx.py,v 1.62 2004/05/21 14:41:32 jdh2358 Exp $'
 
 import sys, os, os.path, math, StringIO
 
@@ -216,8 +285,10 @@ except:
     sys.exit()    
 
 wxapp = wxPySimpleApp()
-wxapp.SetExitOnFrameDelete(False)
-wxInitAllImageHandlers()
+wxapp.SetExitOnFrameDelete(True)
+
+#!!! this is the call that is causing the exception swallowing !!!
+#wxInitAllImageHandlers()
 
 def DEBUG_MSG(string, lvl=3, o=None):
     if lvl >= _DEBUG:
@@ -252,10 +323,11 @@ from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
      FigureCanvasBase, FigureManagerBase, error_msg
 from matplotlib._matlab_helpers import Gcf
 from matplotlib.artist import Artist
+from matplotlib.cbook import exception_to_str
 from matplotlib.figure import Figure
 from matplotlib.text import _process_text_args, Text
-from matplotlib.transforms import Transform, Bound1D, Bound2D
-from matplotlib import get_data_path
+
+from matplotlib import rcParams
 
 
 # the true dots per inch on the screen; should be display dependent
@@ -292,17 +364,32 @@ class RendererWx(RendererBase):
     #describes the colour and weight of any lines drawn, and a wxBrush
     #which describes the fill colour of any closed polygon.
 
-    fontweights = {'normal'     : wxNORMAL,
-                   'bold'       : wxBOLD,
-                   'heavy'      : wxBOLD,
-                   'light'      : wxLIGHT,
-                   'ultrabold'  : wxBOLD,
-                   'ultralight' : wxLIGHT }
-                   
-    fontangles = { 'italic'  : wxITALIC,
-                   'normal'  : wxNORMAL,
-                   'oblique' : wxSLANT }
-                   
+
+    fontweights = {
+        100          : wxLIGHT,
+        200          : wxLIGHT,
+        300          : wxLIGHT,
+        400          : wxNORMAL,
+        500          : wxNORMAL,
+        600          : wxNORMAL,
+        700          : wxBOLD,
+        800          : wxBOLD,
+        900          : wxBOLD,
+        'ultralight' : wxLIGHT,
+        'light'      : wxLIGHT,
+        'normal'     : wxNORMAL,
+        'medium'     : wxNORMAL,
+        'semibold'   : wxNORMAL,
+        'bold'       : wxBOLD,
+        'heavy'      : wxBOLD,
+        'ultrabold'  : wxBOLD,
+        'black'      : wxBOLD
+        }
+    fontangles = {
+        'italic'  : wxITALIC,
+        'normal'  : wxNORMAL,
+        'oblique' : wxSLANT }
+
     # wxPython allows for portable font styles, choosing them appropriately
     # for the target platform. Map some standard font names to the portable
     # styles
@@ -326,43 +413,33 @@ class RendererWx(RendererBase):
         self.bitmap = bitmap
         self.fontd = {}
         self.dpi = dpi
-        #self.gc = None
+        self.gc = None
 
-    def compute_text_offsets(self, t, gc=None):
+    def flipy(self):
+        return True
+
+    def offset_text_height(self):
+        return True
+
+    def get_text_width_height(self, s, prop, ismath):
         """
-        Return the (x,y) offsets to adjust for the alignment
-        specifications.
+        get the width and height in display coords of the string s
+        with FontPropertry prop
         """
+        #return 1, 1
 
-        DEBUG_MSG("_compute_offsets()", 1, self)
-        # Identical to GTK version
 
-        halign = t.get_horizontalalignment()
-        valign = t.get_verticalalignment()
-        font = self.get_wx_font(t)
+        if self.gc is None: gc = self.new_gc()
+        font = self.get_wx_font(s, prop)
+        self.gc.SetFont(font)
+        w, h = self.gc.GetTextExtent(s)
 
-        if gc is None: gc = self.new_gc()
-        w, h = gc.GetTextExtent(t.get_text())
+        return w, h
 
-        if t.get_rotation()=='vertical':
-            w, h = h, w
-            if halign=='center': offsetx = -w/2
-            elif halign=='right': offsetx = -w
-            else: offsetx = 0
+    def get_canvas_width_height(self):
+        'return the canvas width and height in display coords'
+        return self.width, self.height
 
-            if valign=='center': offsety = -h/2
-            elif valign=='top': offsety = 0
-            else: offsety = -h
-        else:
-            if halign=='center': offsetx = -w/2
-            elif halign=='right': offsetx = -w
-            else: offsetx = 0
-
-            if valign=='center': offsety = h/2
-            elif valign=='top': offsety = 0
-            else: offsety = h
-
-        return (offsetx, offsety)
     
     def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):
         """
@@ -454,32 +531,29 @@ class RendererWx(RendererBase):
         gc.DrawPoint(int(x), self.height - int(y))
         gc.unselect()
 
-    def draw_text(self, gc, x, y, t):
+    def draw_text(self, gc, x, y, s, prop, angle, ismath):
         """
         Render the matplotlib.text.Text instance
         None)
         """
+
         DEBUG_MSG("draw_text()", 1, self)
         gc.select()
-        if t.get_text()=='': return
 
-        font = self.get_wx_font(t)
+        font = self.get_wx_font(s, prop)
         gc.SetFont(font)
         assert gc.Ok(), "wxMemoryDC not OK to use"
 
-        ox, oy = self.compute_text_offsets(t, gc)
-        xox = int(x+ox)
-        yoy = int(y+oy)
+        w, h = self.get_text_width_height(s, prop, ismath)
+        x = int(x)
+        y = int(y-h)
 
-        if y+oy<0:
-            print >>sys.stderr, 'Warning: text label "%s" is outside window extent' % t.get_text()
-        if x+ox<0:
-            print >>sys.stderr, 'Warning: text label "%s" is outside window extent' % t.get_text()        
-
-        if t.get_rotation() == 'vertical':
-            gc.DrawRotatedText(t.get_text(), xox, self.height-yoy, 90)
+        if angle!=0:
+            try: gc.DrawRotatedText(s, x, y, angle)
+            except:
+                print exception_to_str('WX rotated text failed')
         else:
-            gc.DrawText(t.get_text(), xox, self.height-yoy)
+            gc.DrawText(s, x, y)
         gc.unselect()
 
     def new_gc(self):
@@ -502,68 +576,46 @@ class RendererWx(RendererBase):
         assert self.gc != None, "gc must be defined"
         return self.gc
             
-    def get_text_extent(self, t, gc=None):
-        """
-        Return the ink extent of the text as Bound2D instance
-        """
 
-        DEBUG_MSG("get_text_extent(t)", 1, self)
-        x, y = t.get_xy_display()
-        font = self.get_wx_font(t)
-        if gc is None: gc = self.new_gc()
-        gc.select()
-        w, h = gc.GetTextExtent(t.get_text())
-        w = self.points_to_pixels(w)
-        h = self.points_to_pixels(h)
-        
-
-        ox, oy = self.compute_text_offsets(t, gc)
-        ox = self.points_to_pixels(ox)
-        oy = self.points_to_pixels(oy)
-
-        l, b = x + ox, y + oy - h
-        if t.get_rotation() == 'vertical':
-            b = b + h
-            w, h = h, w
-        gc.unselect()
-        return Bound2D(l,b,w,h)
-
-    def get_wx_font(self, t):
+    def get_wx_font(self, s, prop):
         """
         Return a wx font.  Cache instances in a font dictionary for
         efficiency
         """
         DEBUG_MSG("get_wx_font()", 1, self)
 
-        props = t.get_prop_tup()
-        font = self.fontd.get(props)
-        if font is not None: return font
 
+        key = hash(prop)
+        fontprop = prop
+        fontname = fontprop.get_name()
+
+        font = self.fontd.get(key)
+        if font is not None:
+            #print 'returning cached'
+            return font
 
         # Allow use of platform independent and dependent font names
         try:
-            wxFontname = self.fontnames[t.get_fontname()]
+            wxFontname = self.fontnames[fontname]
             wxFacename = '' # Empty => wxPython chooses based on wx_fontname
         except KeyError:
             wxFontname = wxSWISS
-            wxFacename = t.get_fontname()
+            wxFacename = fontname
 
         # Font colour is determined by the active wxPen
         # TODO: It may be wise to cache font information
-        scale = int(math.ceil(t.dpi.get() / PIXELS_PER_INCH))
-        DEBUG_MSG('get_wx_font(): font:%s size:%d text:%s' %
-                  (t.get_fontname(), t.get_fontsize()*scale, t.get_text()),
-                  1, self) 
-        font = wxFont(t.get_fontsize() * scale,             # Size
-                      wxFontname,                           # 'Generic' name
-                      self.fontangles[t.get_fontangle()],   # Angle
-                      self.fontweights[t.get_fontweight()], # Weight
-                      False,                                # Underline
-                      wxFacename)                           # Platform font name
-                            
+        size = self.points_to_pixels(fontprop.get_size_in_points())
+
+        
+        font = wxFont(int(size+0.5),             # Size
+                      wxFontname,                # 'Generic' name
+                      self.fontangles[fontprop.get_style()],   # Angle
+                      self.fontweights[fontprop.get_weight()], # Weight
+                      False,                     # Underline
+                      wxFacename)                # Platform font name
 
         # cache the font and gc and return it
-        self.fontd[props] = font
+        self.fontd[key] = font
 
         return font
     
@@ -580,7 +632,7 @@ class RendererWx(RendererBase):
         convert point measures to pixes using dpi and the pixels per
         inch of the display
         """
-        return points*PIXELS_PER_INCH/72.0*self.dpi.get()/72.0
+        return points*(PIXELS_PER_INCH/72.0*self.dpi.get()/72.0)
 
 class GraphicsContextWx(GraphicsContextBase, wxMemoryDC):
     """
@@ -622,6 +674,7 @@ class GraphicsContextWx(GraphicsContextBase, wxMemoryDC):
             GraphicsContextWx._lastWxDC.SelectObject(wxNullBitmap)
         #print >>sys.stderr, "SelectObject(%s)" % str(bitmap)
         self.SelectObject(bitmap)
+        self.bitmap = bitmap
         self.SetPen(wxPen('BLACK', 1, wxSOLID))
         self._style = wxSOLID
         self.renderer = renderer    
@@ -631,17 +684,18 @@ class GraphicsContextWx(GraphicsContextBase, wxMemoryDC):
         """
         Select the current bitmap into this wxDC instance
         """
-        pass
-        #self.SelectObject(self.bitmap)
-        #self.IsSelected = True
+
+        if sys.platform=='win32':
+            self.SelectObject(self.bitmap)
+            self.IsSelected = True
 
     def unselect(self):
         """
         Select a Null bitmasp into this wxDC instance
         """
-        pass
-        #self.SelectObject(wxNullBitmap)
-        #self.IsSelected = False
+        if sys.platform=='win32':
+            self.SelectObject(wxNullBitmap)
+            self.IsSelected = False
 
     def set_clip_rectangle(self, rect):
         """
@@ -651,8 +705,14 @@ class GraphicsContextWx(GraphicsContextBase, wxMemoryDC):
         #print 'set clip', rect
         self.select()
         l,b,w,h = rect
-        self.SetClippingRegion(int(l), self.renderer.height - int(b+h),
-                               int(w), int(h))
+        # this appears to be version dependent'
+        if hasattr(self, 'SetClippingRegionXY'):
+            clipfunc = getattr(self, 'SetClippingRegionXY')
+        else:
+            clipfunc = getattr(self, 'SetClippingRegion')            
+        
+        clipfunc(int(l), self.renderer.height - int(b+h),
+                 int(w), int(h))
         self.unselect()
 
     def set_foreground(self, fg):
@@ -790,22 +850,22 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
           EVT_SIZE  (Resize event)
           EVT_PAINT (Paint event)
         """
+
         FigureCanvasBase.__init__(self, figure)
         # Set preferred window size hint - helps the sizer (if one is
         # connected)
-        w = int(math.ceil(figure.bbox.x.interval()))
-        h = int(math.ceil(figure.bbox.y.interval()))
+        l,b,w,h = figure.bbox.get_bounds()
+        w = int(math.ceil(w))
+        h = int(math.ceil(h))
 
         wxPanel.__init__(self, parent, id, size=wxSize(w, h))
         # Create the drawing bitmap
         self.bitmap = wxEmptyBitmap(w, h)
         DEBUG_MSG("__init__() - bitmap w:%d h:%d" % (w,h), 2, self)
-        
         # TODO: Add support for 'point' inspection and plot navigation.
         self._isRealized = False
         self._isConfigured = False
-        self._printQueued = []
-		
+        self._printQued = []		
         # Event handlers
         EVT_SIZE(self, self._onSize)
         EVT_PAINT(self, self._onPaint)
@@ -829,8 +889,8 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         previously defined renderer if none is specified.
         """
         DEBUG_MSG("draw()", 1, self)
-        renderer = RendererWx(self.bitmap, self.figure.dpi)
-        self.figure.draw(renderer)
+        self.renderer = RendererWx(self.bitmap, self.figure.dpi)
+        self.figure.draw(self.renderer)
         
         
     def gui_repaint(self):
@@ -853,6 +913,7 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         """
         Render the figure to hardcopy
         """
+
         DEBUG_MSG("print_figure()", 1, self)
         # Save state information, and set save DPI
 
@@ -873,21 +934,24 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
             for a in self.figure.axes: a.resize()
             return
 
+        if not self._isRealized:
+            self._printQued.append((filename, dpi, facecolor, edgecolor))
+            return
+
         origfacecolor = self.figure.get_facecolor()
         origedgecolor = self.figure.get_edgecolor()
+        origDPI      = self.figure.dpi.get()
+
+        self.figure.dpi.set(dpi)        
         self.figure.set_facecolor(facecolor)
         self.figure.set_edgecolor(edgecolor)
 
-        origBounds   = self.figure.bbox.get_bounds()
-        origDPI      = self.figure.dpi.get()
-        origRenderer = self.figure.renderer
+
         origBitmap   = self.bitmap
 		
-        dpi = 72.0/PIXELS_PER_INCH*dpi
-        self.figure.dpi.set(dpi)        
-        width = int(self.figure.figsize[0]*dpi)
-        height = int(self.figure.figsize[1]*dpi)
-        self.figure.bbox.set_bounds(0, 0, width, height)
+        l,b,width,height = self.figure.bbox.get_bounds()
+        width = int(math.ceil(width))
+        height = int(math.ceil(height))
 
         # Now create a bitmap and draw onto it
         DEBUG_MSG('print_figure(): bitmap w:%d h:%d' % (width, height), 2, self)
@@ -898,15 +962,12 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         renderer = RendererWx(self.bitmap, self.figure.dpi)
 
         gc = renderer.new_gc()
-        for a in self.figure.axes: a.resize()
 
         self.figure.draw(renderer)
 
         self.figure.set_facecolor(origfacecolor)
         self.figure.set_edgecolor(origedgecolor)
         self.figure.dpi.set(origDPI)
-        self.figure.bbox.set_bounds(*origBounds)
-        for a in self.figure.get_axes(): a.resize()
 
         # Now that we have rendered into the bitmap, save it
         # to the appropriate file type and clean up
@@ -925,6 +986,7 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         # Restore everything to normal
         self.bitmap = origBitmap
 
+
         self.draw()
         self.Refresh()
             
@@ -935,6 +997,11 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         """
         DEBUG_MSG("realize()", 1, self)
         self._isRealized = True
+        for fname, dpi, facecolor, edgecolor in self._printQued:
+            self.print_figure(fname, dpi, facecolor, edgecolor)
+        self._printQued = []
+
+
 
     def _onPaint(self, evt):
         """
@@ -947,11 +1014,18 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         # Render to the bitmap
         self.draw()
         # Must use wxPaintDC during paint event
-        w = int(math.ceil(self.figure.bbox.x.interval()))
-        h = int(math.ceil(self.figure.bbox.y.interval()))
 
-        memDC = self.figure.renderer.new_gc()
-        memDC.select()
+        l,b,w,h = self.figure.bbox.get_bounds()
+        w = int(math.ceil(w))
+        h = int(math.ceil(h))
+
+        
+        # I decoupled this from GraphicsContextWx so it would play
+        # nice with wxagg
+        memDC = wxMemoryDC()
+        memDC.SelectObject(self.bitmap)
+        memDC.SetPen(wxPen('BLACK', 1, wxSOLID))
+
         drawDC=wxPaintDC(self)
         
         drawDC.BeginDrawing()
@@ -959,8 +1033,6 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         drawDC.Blit(0, 0, w, h, memDC, 0, 0)
         #drawDC.DrawBitmap(self.bitmap, 0, 0)
         drawDC.EndDrawing() 
-        #self.bitmap.SaveFile('tmp.bmp', wxBITMAP_TYPE_BMP)
-        memDC.unselect()
         evt.Skip()
         
     def _onSize(self, evt):
@@ -980,13 +1052,12 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         
         # Scale the displayed image (but don't update self.figsize)
         if not self._isConfigured:
-            self._orig_width, self._orig_height = self._width, self._height
             self._isConfigured = True
 
-        self.figure.bbox.set_bounds(0, 0, self._width, self._height)
-        scalew = int(math.ceil(self._width / self._orig_width))
-        scaleh = int(math.ceil(self._height / self._orig_height))
-        for a in self.figure.axes: a.resize()
+        dpival = self.figure.dpi.get()
+        winch = self._width/dpival
+        hinch = self._height/dpival
+        self.figure.set_figsize_inches(winch, hinch)
 
         if self._isRealized:
             self.draw()
@@ -1049,20 +1120,23 @@ def show():
     """
 
     DEBUG_MSG("show()", 3, None)
+
     for figwin in Gcf.get_all_fig_managers():
         figwin.frame.Show()
         figwin.canvas.realize()
         figwin.canvas.draw()
-    if not matplotlib.is_interactive(): wxapp.MainLoop()
 
-def new_figure_manager(num, *args):
+    if not matplotlib.is_interactive():
+        wxapp.MainLoop()
+
+def new_figure_manager(num, *args, **kwargs):
     """
     Create a new figure manager instance
     """
     # in order to expose the Figure constructor to the matlab
     # interface we need to create the figure here
     DEBUG_MSG("new_figure_manager()", 3, None)
-    fig = Figure(*args)
+    fig = Figure(*args, **kwargs)
     frame = FigureFrameWx(num, fig)
     figmgr = frame.get_figure_manager()
     if matplotlib.is_interactive():
@@ -1084,30 +1158,41 @@ class FigureFrameWx(wxFrame):
         DEBUG_MSG("__init__()", 1, self)
         self.num = num
 
-
-        self.canvas = FigureCanvasWx(self, -1, fig)
+        self.canvas = self.get_canvas(fig)
         self.SetStatusBar(StatusBarWx(self))
         self.sizer = wxBoxSizer(wxVERTICAL)
         self.sizer.Add(self.canvas, 1, wxTOP | wxLEFT | wxEXPAND)
         # By adding toolbar in sizer, we are able to put it at the bottom
         # of the frame - so appearance is closer to GTK version
+
         self.toolbar = NavigationToolbarWx(self.canvas, True)
         self.toolbar.Realize()
-        # On Windows platform, default window size is incorrect, so set
-        # toolbar width to figure width.
-        tw, th = self.toolbar.GetSizeTuple()
-        fw, fh = self.canvas.GetSizeTuple()
-        self.toolbar.SetSize(wxSize(fw, th))
-        self.sizer.Add(self.toolbar, 0, wxLEFT | wxEXPAND)
+        if wxPlatform == '__WXMAC__':
+            # Mac platform (OSX 10.3, MacPython) does not seem to cope with
+            # having a toolbar in a sizer. This work-around gets the buttons
+            # back, but at the expense of having the toolbar at the top
+            self.SetToolBar(self.toolbar)
+        else:
+            # On Windows platform, default window size is incorrect, so set
+            # toolbar width to figure width.
+            tw, th = self.toolbar.GetSizeTuple()
+            fw, fh = self.canvas.GetSizeTuple()
+            # By adding toolbar in sizer, we are able to put it at the bottom
+            # of the frame - so appearance is closer to GTK version.
+            # As noted above, doesn't work for Mac.
+            self.toolbar.SetSize(wxSize(fw, th))
+            self.sizer.Add(self.toolbar, 0, wxLEFT | wxEXPAND)
         self.SetSizer(self.sizer)
         self.Fit()
-        # By adding toolbar in sizer, we are able to put it at the bottom
-        # of the frame - so appearance is closer to GTK version
+
         self.figmgr = FigureManagerWx(self.canvas, num, self)
 
         # Event handlers
         EVT_CLOSE(self, self._onClose)
-        
+
+    def get_canvas(self, fig):
+        return FigureCanvasWx(self, -1, fig)
+
     def get_figure_manager(self):
         DEBUG_MSG("get_figure_manager()", 1, self)
         return self.figmgr
@@ -1142,9 +1227,9 @@ class FigureManagerWx(FigureManagerBase):
         self.tb.update()
         return a
         
-    def add_axes(self, rect, axisbg):
+    def add_axes(self, rect, **kwargs):
         DEBUG_MSG("add_axes()", 1, self)
-        a = FigureManagerBase.add_axes(self, rect, axisbg)
+        a = FigureManagerBase.add_axes(self, rect, **kwargs)
         self.tb.update()
         return a
     
@@ -1188,7 +1273,7 @@ def _load_bitmap(filename):
     Returns a wxBitmap object
     """
 
-    basedir = get_data_path()
+    basedir = rcParams['datapath']
 
     bmpFilename = os.path.normpath(os.path.join(basedir, filename))
     if not os.path.exists(bmpFilename):
@@ -1336,6 +1421,7 @@ class NavigationToolbarWx(wxToolBar):
         """
         DEBUG_MSG("_create_controls()", 1, self)
         # Need the following line as Windows toolbars default to 15x16
+
         self.SetToolBitmapSize(wxSize(16,16))
         self.AddSimpleTool(_NTB_X_PAN_LEFT, _load_bitmap('stock_left.xpm'),
                            'Left', 'Scroll left')
@@ -1358,9 +1444,6 @@ class NavigationToolbarWx(wxToolBar):
         self.AddSimpleTool(_NTB_SAVE, _load_bitmap('stock_save_as.xpm'),
                            'Save', 'Save plot contents as images')
         self.AddSeparator()
-        if can_kill:
-            self.AddSimpleTool(_NTB_CLOSE, _load_bitmap('stock_close.xpm'),
-                               'Close', 'Close plot window')        # Events:
         EVT_TOOL(self, _NTB_X_PAN_LEFT, self._onLeftScroll)
         EVT_TOOL(self, _NTB_X_PAN_RIGHT, self._onRightScroll)
         EVT_TOOL(self, _NTB_X_ZOOMIN, self._onXZoomIn)
@@ -1505,7 +1588,7 @@ class NavigationToolbarWx(wxToolBar):
             dirname  = dlg.GetDirectory()
             filename = dlg.GetFilename()
             DEBUG_MSG('Save file dir:%s name:%s' % (dirname, filename), 3, self)
-        self.canvas.print_figure(os.path.join(dirname, filename))
+            self.canvas.print_figure(os.path.join(dirname, filename))
         
     def _onClose(self, evt):
         self.GetParent().Destroy()
