@@ -334,7 +334,7 @@ def pieces(seq, num=2):
    start = 0
    while 1:
       item = seq[start:start+num]
-      if not item: break
+      if not len(item): break
       yield item
       start += num
 
@@ -344,3 +344,8 @@ def exception_to_str(s = None):
    if s is not None: print >>sh, s
    traceback.print_exc(file=sh)
    return sh.getvalue()
+
+
+def warn(s):
+    'issue a warning'
+    print >> sys.stderr, s
