@@ -14,12 +14,17 @@ Plotting commands
   cohere   - make a plot of coherence
   csd      - make a plot of cross spectral density
   errorbar - make an errorbar graph
+  figlegend - make legend on the figure rather than the axes
   figure   - create or change active figure
+  fill     - make filled polygons
   gca      - return the current axes
   gcf      - return the current figure
   get      - get a handle graphics property
   hist     - make a histogram
+  hold     - set the axes hold state
+  legend   - make an axes legend
   loglog   - a log log plot
+  imshow   - plot image data
   pcolor   - make a pseudocolor plot
   plot     - make a line plot
   psd      - make a plot of power spectral density
@@ -29,7 +34,10 @@ Plotting commands
   semilogx - log x axis
   semilogy - log y axis
   show     - show the figures
+  specgram - a spectrogram plot
+  stem     - make a stem plot
   subplot  - make a subplot (numrows, numcols, axesnum)
+  table    - add a table to the plot
   text     - add some text at location x,y to the current axes
   title    - add a title to the current axes
   xlabel   - add an xlabel to the current axes
@@ -99,9 +107,11 @@ Time series analysis
 Other
 
   angle     - the angle of a complex array
+  load     - load ASCII data into array
   polyfit   - fit x, y to an n-th order polynomial
   polyval   - evaluate an n-th order polynomial
   roots     - the roots of the polynomial coefficients in p
+  save      - save an array to an ASCII file
   trapz     - trapezoidal integration
 
 
@@ -114,10 +124,8 @@ the exception of those in mlab.py provided by matplotlib.
 
 # bring all the MLab and mlab symbols in so folks can import them from
 # matplotlib.matlab in one fell swoop
-from Numeric import *
-from MLab import *
+from numerix import *
 from mlab import *
-from FFT import fft
 
 import _matlab_helpers
 import mlab  #so I can override hist, psd, etc...
@@ -126,7 +134,9 @@ from axes import Axes
 from backends import new_figure_manager, error_msg, \
      draw_if_interactive, show
 from cbook import flatten, is_string_like, exception_to_str
+from colors import ColormapJet
 from figure import Figure
+from matplotlib import rcParams, get_backend
 
 def colors():
     """
@@ -168,32 +178,39 @@ def colors():
 def plotting():
     """
     Plotting commands
-
     axes     - Create a new axes
     axis     - Set or return the current axis limits
     bar      - make a bar chart
-    cla      - clear the current axes    
-    clf      - clear the current figure window
-    close    - close the current figure window
+    cla      - clear current axes
+    clf      - clear a figure window
+    close    - close a figure window
     cohere   - make a plot of coherence
     csd      - make a plot of cross spectral density
     errorbar - make an errorbar graph
+    figlegend - add a legend to the figure
     figure   - create or change active figure
+    fill     - make filled polygons
     gca      - return the current axes
     gcf      - return the current figure
     get      - get a handle graphics property
     hist     - make a histogram
+    hold     - set the hold state on current axes
+    legend   - add a legend to the axes
     loglog   - a log log plot
+    imshow   - plot image data
     pcolor   - make a pseudocolor plot
     plot     - make a line plot
     psd      - make a plot of power spectral density
     savefig  - save the current figure
     scatter  - make a scatter plot
+    set      - set a handle graphics property
     semilogx - log x axis
     semilogy - log y axis
-    set      - set a handle graphics property
     show     - show the figures
+    specgram - a spectrogram plot
+    stem     - make a stem plot
     subplot  - make a subplot (numrows, numcols, axesnum)
+    table    - add a table to the axes
     text     - add some text at location x,y to the current axes
     title    - add a title to the current axes
     xlabel   - add an xlabel to the current axes
@@ -211,11 +228,12 @@ def get_current_fig_manager():
         figManager = _matlab_helpers.Gcf.get_active()
     return figManager
 
-def get_plot_commands(): return ['axis', 'axes', 'subplot', 'plot',
-    'set', 'get', 'title', 'xlabel', 'ylabel', 'text', 'scatter',
-    'errorbar', 'bar', 'hist', 'figure', 'gca', 'gcf', 'close',
-    'pcolor', 'loglog', 'semilogx', 'semilogy', 'psd', 'csd', 'cohere', 'clf', 'cla']
-
+def get_plot_commands(): return ('axes', 'axis', 'bar', 'cla', 'clf',
+'close', 'cohere', 'csd', 'errorbar', 'figlegend', 'figure', 'fill',
+'gca', 'gcf', 'get', 'hist', 'hold', 'imshow', 'legend', 'loglog', 'pcolor',
+'plot', 'psd', 'savefig', 'scatter', 'set', 'semilogx', 'semilogy',
+'show', 'specgram', 'stem', 'subplot', 'table', 'text', 'title',
+'xlabel', 'ylabel')
 
 def raise_msg_to_str(msg):
     """msg is a return arg from a raise.  Join with new lines"""
@@ -291,13 +309,12 @@ def axes(*args, **kwargs):
 
     if isinstance(arg, Axes):
         get_current_fig_manager().set_current_axes(arg)
-        return arg
+        ret = arg
     else:
         rect = arg
-        bg = kwargs.get('axisbg', 'w')
-        ret = get_current_fig_manager().add_axes(rect, bg)
-        return ret
-
+        ret = get_current_fig_manager().add_axes(rect, **kwargs)
+    draw_if_interactive()
+    return ret
 
 def bar(*args, **kwargs):
     """
@@ -316,6 +333,7 @@ def bar(*args, **kwargs):
     on the bar chart
 
     color specifies the color of the bar
+    ecolor specifies the color of any errorbar
 
     capsize determines the length in points of the error bar caps
 
@@ -331,7 +349,7 @@ def bar(*args, **kwargs):
     except Exception, msg:
         s = exception_to_str(msg)
         error_msg(s)
-        raise RuntimeError('Could not exectute bar')
+        raise RuntimeError(msg)
     draw_if_interactive()
     return patches
 
@@ -452,7 +470,7 @@ def csd(x, y, NFFT=256, Fs=2, detrend=mlab.detrend_none,
         draw_if_interactive()
         return ret
 
-def errorbar(x, y, yerr=None, xerr=None, fmt='b-', capsize=3):
+def errorbar(x, y, yerr=None, xerr=None, fmt='b-', ecolor='k', capsize=3):
     """
     Plot x versus y with error deltas in yerr and xerr.
     Vertical errorbars are plotted if yerr is not None
@@ -462,15 +480,16 @@ def errorbar(x, y, yerr=None, xerr=None, fmt='b-', capsize=3):
         a rank-0, Nx1 Numpy array  - symmetric errorbars +/- value
         an N-element list or tuple - symmetric errorbars +/- value
         a rank-1, Nx2 Numpy array  - asymmetric errorbars -column1/+column2
-    
+        
     fmt is the plot format symbol for y
+    ecolor is the errorbar color specifier
 
     Return value is a length 2 tuple.  The first element is a list of
     y symbol lines.  The second element is a list of error bar lines.
 
     capsize is the size of the error bar caps in points
     """
-    try: ret =  gca().errorbar(x, y, yerr, xerr, fmt, capsize)
+    try: ret =  gca().errorbar(x, y, yerr, xerr, fmt, ecolor, capsize)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
@@ -478,8 +497,37 @@ def errorbar(x, y, yerr=None, xerr=None, fmt='b-', capsize=3):
         draw_if_interactive()
         return ret
 
-def figure(num=1, figsize=(8,6), dpi=72, facecolor=0.75, edgecolor='w'):
+    
+def figlegend(handles, labels, loc):
     """
+    Place a legend in the figure.  Labels are a sequence of
+    strings, handles is a sequence of line or patch instances, and
+    loc can be a string or an integer specifying the legend
+    location
+
+    USAGE: 
+      legend( (line1, line2, line3),
+              ('label1', 'label2', 'label3'),
+              'upper right')
+
+    See help(legend) for information about the location codes
+    """
+    l=  gcf().legend(handles, labels, loc)
+    draw_if_interactive()
+    return l
+    
+
+def figure(num=1,
+           figsize   = rcParams['figure.figsize'],
+           dpi       = rcParams['figure.dpi'],
+           facecolor = rcParams['figure.facecolor'],
+           edgecolor = rcParams['figure.edgecolor'],
+           frameon = True,
+           ):
+    """
+    figure(num = 1, figsize=(8, 6), dpi=80, facecolor='w', edgecolor='k')
+
+
     Create a new figure and return a handle to it
 
     If figure(num) already exists, make it active and return the
@@ -487,6 +535,13 @@ def figure(num=1, figsize=(8,6), dpi=72, facecolor=0.75, edgecolor='w'):
 
       figure(1)
 
+    figsize - width in height x inches
+    dpi     - resolution
+    facecolor - the background color    
+    edgecolor - the border color
+
+    rcParams gives the default values from the .matplotlibrc file
+
     """
     if num==0:
         error_msg('Figure number can not be 0.\n' + \
@@ -494,11 +549,60 @@ def figure(num=1, figsize=(8,6), dpi=72, facecolor=0.75, edgecolor='w'):
 
     figManager = _matlab_helpers.Gcf.get_fig_manager(num)
     if figManager is None:
-        figManager = new_figure_manager(num, figsize, dpi, facecolor, edgecolor)
+        if get_backend()=='PS':  dpi = 72
+        figManager = new_figure_manager(num, figsize, dpi, facecolor, edgecolor, frameon)
         _matlab_helpers.Gcf.set_active(figManager)
     
     return figManager.canvas.figure
 
+def fill(*args, **kwargs):
+    """
+    plot filled polygons.  *args is a variable length argument,
+    allowing for multiple x,y pairs with an optional color format
+    string.  For example, all of the following are legal, assuming a
+    is the Axis instance:
+
+      fill(x,y)            # plot polygon with vertices at x,y
+      fill(x,y, 'b' )      # plot polygon with vertices at x,y in blue
+
+    An arbitrary number of x, y, color groups can be specified, as in 
+      fill(x1, y1, 'g', x2, y2, 'r')  
+
+    Return value is a list of patches that were added
+
+    The following color strings are supported
+
+      b  : blue
+      g  : green
+      r  : red
+      c  : cyan
+      m  : magenta
+      y  : yellow
+      k  : black 
+      w  : white
+
+    The kwargs that are can be used to set line properties (any
+    property that has a set_* method).  You can use this to set edge
+    color, face color, etc.
+
+    Example code:
+
+    from matplotlib.matlab import *
+    t = arange(0.0, 1.01, 0.01)
+    s = sin(2*2*pi*t)
+
+    fill(t, s, 'r')
+    grid(True)
+    show()
+
+    """
+    try: patch =  gca().fill(*args, **kwargs)
+    except Exception, msg:
+        s = exception_to_str(msg)
+        error_msg(s)
+        raise RuntimeError('Could not exectute fill')
+    draw_if_interactive()
+    return patch
 
 def gca():
     """
@@ -547,8 +651,9 @@ def grid(b):
     Set the figure grid to be on or off (b is a boolean)
     """
     gca().grid(b)
-
-def hist(x, bins=10, noplot=0, normed=0):
+    draw_if_interactive()
+    
+def hist(x, bins=10, noplot=0, normed=0, bottom=0):
     """
     Compute the histogram of x.  bins is either an integer number of
     bins or a sequence giving the bins.  x are the data to be binned.
@@ -567,7 +672,7 @@ def hist(x, bins=10, noplot=0, normed=0):
     if noplot: return mlab.hist(x, bins, normed)
     else:
         try:
-            ret = gca().hist(x, bins, normed)
+            ret = gca().hist(x, bins, normed, bottom)
         except ValueError, msg:
             msg = raise_msg_to_str(msg)
             error_msg(msg)
@@ -576,7 +681,18 @@ def hist(x, bins=10, noplot=0, normed=0):
     return ret
 
     
+def hold(b=None):
+    """
+    Set the hold state.  If hold is None (default), toggle the
+    hold state.  Else set the hold state to boolean value b.
 
+    Eg
+    hold()      # toggle hold
+    hold(True)  # hold is on
+    hold(False) # hold is off
+    """
+    gca().hold(b)
+    
 def hlines(*args, **kwargs):    
     """
     lines = hlines(self, y, xmin, xmax, fmt='k-')
@@ -598,6 +714,39 @@ def hlines(*args, **kwargs):
     return lines
 
 
+def imshow(*args, **kwargs):
+    """
+    Display the image in array X to current axes.  X must be a
+    float array
+
+    Usage: 
+
+     IMSHOW(X)
+    
+     If X is MxN, assume luminance (grayscale)
+     If X is MxNx3, assume RGB
+     If X is MxNx4, assume RGBA
+
+     IMSHOW(X, cmap)
+     cmap is a colors.Colormap instance used to make a pseudo-color plot
+    
+    An Image instance is returned
+    """
+
+    try: im =  gca().imshow(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+        raise RuntimeError(msg)
+    except RuntimeError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+        raise RuntimeError(msg)
+    draw_if_interactive()
+
+    return im
+
+
 def legend(*args, **kwargs):
     """
     Place a legend on the current axes at location loc.  Labels are a
@@ -624,7 +773,7 @@ def legend(*args, **kwargs):
 
     The LOC location codes are
 
-    The location codes are
+    The LOC location codes are
 
       'best' : 0,          (currently not supported, defaults to upper right)
       'upper right'  : 1,  (default)
@@ -638,12 +787,70 @@ def legend(*args, **kwargs):
       'upper center' : 9,
       'center'       : 10,
 
-    Return value is a sequence of text, line instances that make
-    up the legend
+    If none of these are suitable, loc can be a 2-tuple giving x,y
+    in axes coords, ie,
+
+      loc = 0, 1 is left top
+      loc = 0.5, 0.5 is center, center
+
+      and so on
+
+
+    The legend instance is returned
     """
 
-    gca().legend(*args, **kwargs)
+    ret = gca().legend(*args, **kwargs)
+    draw_if_interactive()
+    return ret
 
+def load(fname):
+    """
+    Load ASCII data from fname into an array and return the array.
+
+    The data must be regular, same number of values in every row
+
+    fname can be a filename or a file handle
+
+    matfile data is not currently supported, but see
+    Nigel Wade's matfile ftp://ion.le.ac.uk/matfile/matfile.tar.gz
+
+    Example usage:
+
+    x,y = load('test.dat')  # data in two columns
+
+    X = load('test.dat')    # a matrix of data
+
+    x = load('test.dat')    # a single column of data
+
+    """
+
+    if is_string_like(fname):
+        fh = file(fname)
+    elif hasattr(fname, seek):
+        fh = fname
+    else:
+        raise ValueError('fname must be a string or file handle')
+    
+    X = []
+    numCols = None
+    for line in fh:
+        line = line.strip()
+        if not len(line): continue
+        row = [float(val) for val in line.split()]
+        thisLen = len(row)
+        if numCols is not None and thisLen != numCols:
+            raise ValueError('All rows must have the same number of columns')
+        X.append(row)
+
+    X = array(X)
+    r,c = X.shape
+    if r==1 or c==1:
+        X.shape = max([r,c]),
+    return X
+
+
+    
+            
 
 def loglog(*args, **kwargs):
     """
@@ -662,7 +869,7 @@ def loglog(*args, **kwargs):
 
 def pcolor(*args, **kwargs):
     """
-    pcolor(C) - make a pseudocolor plot of matrix C
+    pcolor_patch(C) - make a pseudocolor plot of matrix C
 
     pcolor(X, Y, C) - a pseudo color plot of C on the matrices X and Y  
 
@@ -675,7 +882,7 @@ def pcolor(*args, **kwargs):
          pcolor(C, shading='flat')  
          pcolor(X, Y, C, shading='faceted')
 
-    returns a list of patch objects.
+    Return value is a matplotlib.collections.PolyCollection object
 
     Note, the behavior of meshgrid in matlab is a bit
     counterintuitive for x and y arrays.  For example,
@@ -700,7 +907,53 @@ def pcolor(*args, **kwargs):
     and numcols of Z
 
     """
-    return gca().pcolor(*args, **kwargs)
+    ret = gca().pcolor(*args, **kwargs)
+    draw_if_interactive()
+    return ret
+
+def pcolor_classic(*args, **kwargs):
+    """
+    pcolor_classic(C) - make a pseudocolor plot of matrix C
+
+    pcolor_classic(X, Y, C) - a pseudo color plot of C on the matrices X and Y  
+
+    Shading:
+
+      The optional keyword arg shading ('flat' or 'faceted') will
+      determine whether the black grid is drawn around each pcolor
+      square.  Defaul 'faceteted'
+         e.g.,   
+         pcolor_classic(C, shading='flat')  
+         pcolor_classic(X, Y, C, shading='faceted')
+
+    returns a list of patch objects.
+
+    Note, the behavior of meshgrid in matlab is a bit
+    counterintuitive for x and y arrays.  For example,
+
+      x = arange(7)
+      y = arange(5)
+      X, Y = meshgrid(x,y)
+
+      Z = rand( len(x), len(y))
+      pcolor(X, Y, Z)
+
+    will fail in matlab and matplotlib.  You will probably be
+    happy with
+
+      pcolor_classic(X, Y, transpose(Z))
+
+    Likewise, for nonsquare Z,
+
+      pcolor_classic(transpose(Z))
+
+    will make the x and y axes in the plot agree with the numrows
+    and numcols of Z
+
+    """
+    ret = gca().pcolor_classic(*args, **kwargs)
+    draw_if_interactive()
+    return ret
 
 def plot(*args, **kwargs):
     """
@@ -734,6 +987,18 @@ def plot(*args, **kwargs):
       >  : triangle right symbols
       s  : square symbols
       +  : plus symbols
+      x  : cross symbols
+      D  : diamond symbols
+      d  : thin diamond symbols
+      1  : tripod down symbols
+      2  : tripod up symbols
+      3  : tripod left symbols
+      4  : tripod right symbols
+      h  : hexagon symbols
+      H  : rotated hexagon symbols
+      p  : pentagon symbols
+      |  : vertical line symbols
+      _  : horizontal line symbols
 
     The following color strings are supported
 
@@ -746,9 +1011,24 @@ def plot(*args, **kwargs):
       k  : black 
       w  : white
 
-   Line styles and colors are combined in a single format string
-   
+    Line styles and colors are combined in a single format string
 
+    The kwargs that are can be used to set line properties (any
+    property that has a set_* method).  You can use this to set a line
+    label (for auto legends), linewidth, anitialising, marker face
+    color, etc.  Here is an example:
+
+      plot([1,2,3], [1,2,3], 'go-', label='line 1', linewidth=2)
+      plot([1,2,3], [1,4,9], 'rs',  label='line 2')
+      axis([0, 4, 0, 10])
+      legend()
+
+    If you make multiple lines with one plot command, the kwargs apply
+    to all those lines, eg
+
+      plot(x1, y1, x2, y2, antialising=False)
+
+    Neither line will be antialiased.
     """
     
     try: lines =  gca().plot(*args, **kwargs)
@@ -760,6 +1040,29 @@ def plot(*args, **kwargs):
         return lines
 
 
+
+def plot_date(*args, **kwargs):
+    """
+    PLOT_DATE(d, y, converter, fmt='bo', **kwargs)
+
+    d is a sequence of dates; converter is a dates.DateConverter
+    instance that converts your dates to seconds since the epoch for
+    plotting.  y are the y values at those dates.  fmt is a plot
+    format string.  kwargs are passed on to plot.  See plot for more
+    information.
+
+    pass converter = None if your dates are already in epoch format
+    """
+    
+    try: lines =  gca().plot_date(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return lines
+
+
 def psd(x, NFFT=256, Fs=2, detrend=mlab.detrend_none,
         window=mlab.window_hanning, noverlap=0):
     """
@@ -801,8 +1104,40 @@ def psd(x, NFFT=256, Fs=2, detrend=mlab.detrend_none,
         return ret
     
         
+def save(fname, X, fmt='%1.4f'):
+    """
+    Save the data in X to file fname using fmt string to convert the
+    data to strings
 
+    fname can be a filename or a file handle    
 
+    Example usage:
+
+    save('test.out', X)         # X is an array
+    save('test1.out', (x,y,z))  # x,y,z equal sized 1D arrays
+    save('test2.out', x)        # x is 1D
+    save('test3.out', x, fmt='%1.4e')  # use exponential notation
+
+    """
+
+    if is_string_like(fname):
+        fh = file(fname, 'w')
+    elif hasattr(fname, seek):
+        fh = fname
+    else:
+        raise ValueError('fname must be a string or file handle')
+
+    
+    X = asarray(X)
+    origShape = None
+    if len(X.shape)==1:
+        origShape = X.shape
+        X.shape = len(X), 1
+    for row in X:
+        fh.write(' '.join([fmt%val for val in row]) + '\n')
+
+    if origShape is not None:
+        X.shape = origShape
 
 def savefig(*args, **kwargs):
     """
@@ -819,6 +1154,10 @@ def savefig(*args, **kwargs):
     orientation is either 'landscape' or 'portrait' - not supported on
     all backends; currently only on postscript output."""
 
+    for key in ('dpi', 'facecolor', 'edgecolor'):
+        if not kwargs.has_key(key):
+            kwargs[key] = rcParams['savefig.%s'%key]
+
     manager = get_current_fig_manager()
     manager.canvas.print_figure(*args, **kwargs)
 
@@ -845,6 +1184,29 @@ def scatter(*args, **kwargs):
     return patches
 
 
+def scatter_classic(*args, **kwargs):
+    """
+
+    scatter_classic(self, x, y, s=None, c='b'):
+
+    Make a scatter plot of x versus y.  s is a size (in data
+    coords) and can be either a scalar or an array of the same
+    length as x or y.  c is a color and can be a single color
+    format string or an length(x) array of intensities which will
+    be mapped by the colormap jet.        
+
+    If size is None a default size will be used
+    """
+
+    try: patches =  gca().scatter_classic(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+        raise RuntimeError, msg
+    draw_if_interactive()
+    return patches
+
+
 
 def semilogx(*args, **kwargs):
     """
@@ -852,14 +1214,16 @@ def semilogx(*args, **kwargs):
     semilog x are the same as the args to plot.  See help plot for
     more info    
     """
-    try: ret =  gca().semilogx(*args, **kwargs)
+    ret =  gca().semilogx(*args, **kwargs)
+    return ret
+    """
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
     else:
         draw_if_interactive()
         return ret
-
+    """
 
 def semilogy(*args, **kwargs):
     """
@@ -914,13 +1278,65 @@ def set(h, *args, **kwargs):
             func = getattr(o,funcName)        
             try: ret.extend( [func(val)] )
             except ValueError, msg:
-                msg = raise_msg_to_str(msg)
+                msg = exception_to_str(msg)
                 error_msg(msg)
-                raise RuntimeError, msg
+                raise RuntimeError(msg)
         
     draw_if_interactive()
     return [x for x in flatten(ret)]
 
+
+def specgram(x, NFFT=256, Fs=2, detrend=mlab.detrend_none,
+             window=mlab.window_hanning, noverlap=128,
+             cmap = ColormapJet(256)):
+    """
+    Compute a spectrogram of data in x.  Data are split into NFFT
+    length segements and the PSD of each section is computed.  The
+    windowing function window is applied to each segment, and the
+    amount of overlap of each segment is specified with noverlap
+
+    cmap is a a matplotlib.colors.ColorMap
+    
+    See help(psd) for information on the other arguments
+
+    return value is Pxx, freqs, bins, im
+
+    bins are the time points the spectrogram is calculated over
+    freqs is an array of frequencies
+    Pxx is a len(times) x len(freqs) array of power
+    im is a matplotlib image
+    """
+    try: ret =  gca().specgram(x, NFFT, Fs, detrend, window, noverlap, cmap)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+
+def stem(*args, **kwargs):
+    """
+    stem(x, y, linefmt='b-', markerfmt='bo', basefmt='r-')
+
+    A stem plot plots vertical lines (using linefmt) at each x
+    location from the baseline to y, and places a marker there using
+    markerfmt.  A horizontal line at 0 is is plotted using basefmt
+    
+    return value is markerline, stemlines, baseline
+
+    See
+    http://www.mathworks.com/access/helpdesk/help/techdoc/ref/stem.html
+    for details and examples/stem_plot.py for a demo.
+    """
+    try: ret =  gca().stem(*args, **kwargs)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+    
+
 def subplot(*args, **kwargs):
     """
     Create a subplot command, creating axes with
@@ -959,17 +1375,17 @@ def text(x, y, label, fontdict=None, **kwargs):
     fontdict is a dictionary to override the default text properties.
     If fontdict is None, the default is
 
-      'fontsize'            : 9,
+      'fontsize'            : 'x-small',
       'verticalalignment'   : 'bottom',
       'horizontalalignment' : 'left'
 
     **kwargs can in turn be used to override the fontdict, as in
 
-      a.text(x,y,label, fontsize=12)
+      a.text(x,y,label, fontsize='medium')
 
     This command supplies no override dict, and so will have
     'verticalalignment'='bottom' and 'horizontalalignment'='left' but
-    the keyword arg 'fontsize' will create a fontsize of 12
+    the keyword arg 'fontsize' will create a fontsize of medium or 12
 
     The purpose these options is to make it easy for you to create a
     default font theme for your plots by creating a single dictionary,
@@ -982,16 +1398,16 @@ def text(x, y, label, fontdict=None, **kwargs):
           'fontweight'          : 'bold'
           }
 
-        title('My title', fonts, fontsize=12)
-        xlabel('My xlabel', fonts, fontsize=10)
-        ylabel('My ylabel', fonts, fontsize=10)
-        text(12, 20, 'some text', fonts, fontsize=8)
+        title('My title', fonts, fontsize='medium')
+        xlabel('My xlabel', fonts, fontsize='small')
+        ylabel('My ylabel', fonts, fontsize='small')
+        text(12, 20, 'some text', fonts, fontsize='x-small')
 
     The Text defaults are
 
         'color'               : 'k',
         'fontname'            : 'Sans',
-        'fontsize'            : 10,
+        'fontsize'            : 'small',
         'fontweight'          : 'bold',
         'fontangle'           : 'normal',
         'horizontalalignment' : 'left'
@@ -1024,7 +1440,7 @@ def title(s, *args, **kwargs):
 
     Default font override is:
       override = {
-        'fontsize'            : 11,
+        'fontsize'            : 'medium',
         'verticalalignment'   : 'bottom',
         'horizontalalignment' : 'center'
       }
@@ -1065,7 +1481,7 @@ def xlabel(s, *args, **kwargs):
     Default override is
 
       override = {
-          'fontsize'            : 10,
+          'fontsize'            : 'small',
           'verticalalignment'   : 'top',
           'horizontalalignment' : 'center'
           }
@@ -1085,7 +1501,7 @@ def ylabel(s, *args, **kwargs):
     Defaults override is
 
         override = {
-           'fontsize'            : 10,
+           'fontsize'            : 'small',
            'verticalalignment'   : 'center',
            'horizontalalignment' : 'right',
            'rotation'='vertical' : }
